// -*- C++ -*-
/*!
 * @file ECFactory.h
 * @brief ExecutionContext Factory class
 * @date $Date: 2007-12-31 03:08:03 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2007-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: ECFactory.h 1569 2009-11-12 14:36:47Z n-ando $
 *
 */

#ifndef RTC_ECFACTORY_H
#define RTC_ECFACTORY_H

#include <rtm/ExecutionContextBase.h>
#include <string>

namespace RTC 
{
  
  typedef ExecutionContextBase* (*ECNewFunc)();
  typedef void (*ECDeleteFunc)(ExecutionContextBase* ec);
  
  /*!
   * @if jp
   *
   * @brief ExecutionContextpev[g֐
   * 
   * ExecutionContext̃CX^X𐶐邽߂̃ev[g֐B
   *
   * @return ExecutionContextCX^X
   * 
   * @else
   *
   * @brief Template function to create ExecutionContext
   * 
   * Template function to create ExecutionContext's instances.
   *
   * @return Created ExecutionContext's instances
   *
   * @endif
   */
  template <class _New>
  ExecutionContextBase* ECCreate()
  {
    return new _New();
  }
  
  /*!
   * @if jp
   *
   * @brief ExecutionContextjpev[g֐
   * 
   * ExecutionContext̃CX^Xj邽߂̃ev[g֐B
   *
   * @param ec jΏExecutionContext̃CX^X
   *
   * @else
   *
   * @brief Template function to destroy ExecutionContext
   * 
   * Template function to destroy ExecutionContext's instances.
   *
   * @param ec The target ExecutionContext's instances for destruction
   *
   * @endif
   */
  template <class _Delete>
  void ECDelete(ExecutionContextBase* ec)
  {
    delete ec;
  }
  
  /*!
   * @if jp
   * @class ECFactoryBase
   * @brief ECFactoryBase ۃNX
   * 
   * ExecutionContextpFactory̒ۃNXB
   * eExecutionContext𐶐邽߂̋FactoryNX́A
   * ȉ̏z֐̎񋟂Ȃ΂ȂȂB
   *
   * publicC^[tF[XƂĈȉ̂̂񋟂B
   * - name()   : ΏExecutionContext̂̎擾
   * - create() : ExecutionContextCX^X̐
   * - destroy(): ExecutionContextCX^X̔j
   *
   * @since 0.4.0
   *
   * @else
   * @class ECFactoryBase
   * @brief ECFactoryBase abstract class
   * 
   * This is the abstruct Factory classes for ExecutionContext creation.
   * Concrete classes for each ExecutionContext creation must implement 
   * the following pure virtual functions.
   *
   * This class provides the following public interfaces.
   * - name()   : Get names of the target ExecutionContext for creation
   * - create() : Create ExecutionContext's instances
   * - destroy(): Destroy ExecutionContext's instances
   *
   * @since 0.4.0
   *
   * @endif
   */
  class ECFactoryBase
  {
  public:
    /*!
     * @if jp
     *
     * @brief zfXgN^
     * 
     * zfXgN^B
     *
     * @else
     *
     * @brief Virtual destructor
     * 
     * Virtual destructor
     *
     * @endif
     */
    virtual ~ECFactoryBase(void){};
    
    /*!
     * @if jp
     *
     * @brief ΏExecutionContext̎擾pz֐
     * 
     * ΏExecutionContext̖̂擾邽߂̏z֐B
     *
     * @return ΏExecutionContext
     * 
     * @else
     *
     * @brief Pure virtual function to get names of creation target 
     *        ExecutionContext
     * 
     * Pure virtual function to get names of the target ExecutionContext
     * for creation.
     *
     * @return Names of the target ExecutionContext for creation
     *
     * @endif
     */
    virtual const char* name() = 0;
    
    /*!
     * @if jp
     *
     * @brief ExecutionContextpz֐
     * 
     * ExecutionContext̃CX^X𐶐邽߂̏z֐B
     *
     * @return ExecutionContextCX^X
     * 
     * @else
     *
     * @brief Pure virtual function to create ExecutionContext.
     * 
     * Pure virtual function to create ExecutionContext's instances.
     *
     * @return Created ExecutionContext's instances
     *
     * @endif
     */
    virtual ExecutionContextBase* create() = 0;
    
    /*!
     * @if jp
     *
     * @brief ExecutionContextjpz֐
     * 
     * ExecutionContext̃CX^Xj邽߂̏z֐B
     *
     * @param comp jΏۂExecutionContextCX^X
     * 
     * @else
     *
     * @brief Pure virtual function to destroy ExecutionContext.
     * 
     * Pure virtual function to destroy ExecutionContext's instances.
     *
     * @param comp The target ExecutionContext's instances for destruction
     *
     * @endif
     */
    virtual void destroy(ExecutionContextBase* comp) = 0;
  protected:
  };
  
  /*!
   * @if jp
   * @class ECFactoryCXX
   * @brief ECFactoryCXX NX
   * 
   * C++pExecutionContextCX^X𐶐FactoryNXB
   *
   * @since 0.4.0
   *
   * @else
   * @class ECFactoryCXX
   * @brief ECFactoryCXX class
   * 
   * Factory class to create the ExecutionContext's instances for C++.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class ECFactoryCXX
    : public ECFactoryBase
  {
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     * 
     * RXgN^
     *
     * @param name ΏExecutionContext
     * @param new_func ExecutionContextp֐
     * @param delete_func ExecutionContextjp֐
     * 
     * @else
     *
     * @brief Constructor
     * 
     * Constructor
     *
     * @param name Name of the target ExecutionContext for creation
     * @param new_func Function to create ExecutionContext
     * @param delete_func Function to destroy ExecutionContext
     *
     * @endif
     */
    ECFactoryCXX(const char* name,
		 ECNewFunc new_func,
		 ECDeleteFunc delete_func);
    
    /*!
     * @if jp
     *
     * @brief zfXgN^
     * 
     * zfXgN^B
     *
     * @else
     *
     * @brief Virtual destructor
     * 
     * Virtual destructor.
     *
     * @endif
     */
    ~ECFactoryCXX(void);
    
    /*!
     * @if jp
     *
     * @brief ΏExecutionContext̂擾
     * 
     * ΏۂExecutionContext̂擾B
     *
     * @return ΏExecutionContext
     * 
     * @else
     *
     * @brief Get names of the target ExecutionContext for creation
     * 
     * Get names of the target ExecutionContext for creation.
     *
     * @return Names of target ExecutionContext for creation
     *
     * @endif
     */
    virtual const char* name();
    
    /*!
     * @if jp
     *
     * @brief ΏExecutionContextCX^X𐶐
     * 
     * ΏۂExecutionContextNX̃CX^X𐶐B
     *
     * @return ExecutionContextCX^X
     * 
     * @else
     *
     * @brief Create the target ExecutionContext's instances
     * 
     * Create the target ExecutionContext class's instances.
     *
     * @return Created ExecutionContext's instances
     *
     * @endif
     */
    virtual ExecutionContextBase* create();
    
    /*!
     * @if jp
     *
     * @brief ΏExecutionContextCX^Xj
     * 
     * ΏExecutionContextNX̃CX^XjB
     *
     * @param comp jΏExecutionContextCX^X
     * 
     * @else
     *
     * @brief Destroy the target ExecutionContext's instances
     * 
     * Destroy the target ExecutionContext's instances.
     *
     * @param comp The target ExecutionContext's instances to destroy
     *
     * @endif
     */
    virtual void destroy(ExecutionContextBase* comp);
    
  protected:
    /*!
     * @if jp
     * @brief  ΏExecutionContext
     * @else
     * @brief  Names of the target ExecutionContext for creation
     * @endif
     */
    std::string m_name;
    
    /*!
     * @if jp
     * @brief  ΏExecutionContextp֐
     * @else
     * @brief  Function to create the target ExecutionContext
     * @endif
     */
    ECNewFunc m_New;
    
    /*!
     * @if jp
     * @brief  ΏExecutionContextjp֐
     * @else
     * @brief  Function to destroy the target ExecutionContext
     * @endif
     */
    ECDeleteFunc m_Delete;
  };
};
#endif // RTC_ECFACTORY_H
